@@ -1,4 +1,4 @@
-__version__ = "0.0.5"
+__version__ = "0.0.6"
 
 from .core import (
     broadcast_batch,
@@ -16,6 +16,7 @@ from .core import (
     rotx,
     roty,
     rotz,
+    map_range,
     create_random_cameras_on_unit_sphere,
     opengl_c2w_to_opencv_c2w,
     opencv_c2w_to_opengl_c2w,
@@ -48,6 +49,7 @@ from .geometry_advanced import (
 )
 
 from .gsoup_io import (
+    write_to_json,
     save_image,
     save_images,
     save_animation,
@@ -68,6 +70,30 @@ from .image import (
     generate_voronoi_diagram,
     interpolate_single_channel,
     interpolate_multi_channel,
+    image_grid,
+    resize_images_naive,
+    change_brightness,
+    change_contrast
+)
+
+from .video import (
+    get_video_info,
+    get_frame_timestamps,
+    load_video,
+    save_video,
+    reverse_video,
+    compress_video,
+    video_to_images,
+    slice_from_video,
+    VideoReader,
+    FPS
+)
+
+from .procam import (
+    warp_image,
+    generate_gray_code,
+    pix2pix_correspondence,
+    calibrate_procam
 )
 
 from .sphere_trace import (
