@@ -78,16 +78,13 @@ def compose_rt(R: np.array, t: np.array):
 def to_44(mat: np.array):
     """
     converts a 3x4 to a 4x4 matrix by concatenating 0 0 0 1
-    :param mat: 3x4 numpy array
-    :return: 4x4 numpy array
+    :param mat: dimsx3x4 numpy array (dims can be any number of dims including 0)
+    :return: dimsx4x4 numpy array
     """
-    if mat.ndim == 3:
-        assert mat.shape[1:] == (3, 4)
-        to_cat = np.broadcast_to(np.array([0, 0, 0, 1]), (mat.shape[0], 1, 4))
-        new_mat = np.concatenate((mat, to_cat), axis=1)
-    else:
-        assert mat.shape == (3, 4)
-        new_mat = np.concatenate((mat, np.array([0, 0, 0, 1])[None, :]), axis=0)
+    if mat.shape[-2:] != (3, 4):
+        raise ValueError("mat must be 3x4")
+    to_cat = np.broadcast_to(np.array([0, 0, 0, 1]), (*mat.shape[:-2], 1, 4))
+    new_mat = np.concatenate((mat, to_cat), axis=-2)
     return new_mat
 
 def to_34(mat: np.array):
@@ -118,7 +115,7 @@ def look_at_np(from_, to_, up_, openGL=False):
     from_, to_, up_ = broadcast_batch(from_, to_, up_)
     forward = to_ - from_
     forward = forward / np.linalg.norm(forward, axis=-1, keepdims=True)
-    right = np.cross(up_, forward)
+    right = np.cross(forward, up_)
     right = right / np.linalg.norm(right, axis=-1, keepdims=True)
     up = np.cross(forward, right)
     up = up / np.linalg.norm(up, axis=-1, keepdims=True)
@@ -427,4 +424,12 @@ def rotz(a):
     return np.array([[c,-s,0,0],
                       [s,c,0,0],
                       [0,0,1,0],
-                      [0,0,0,1]])
\ No newline at end of file
+                      [0,0,0,1]])
+
+def map_range(x, in_min, in_max, out_min, out_max):
+    """
+    given an input and a range, maps it to a new range
+    """
+    if not in_min <= x <= in_max:
+        raise ValueError("input must be inside range ({} - {})".format(in_min, in_max))
+    return int((x-in_min) * (out_max-out_min) / (in_max-in_min) + out_min)
\ No newline at end of file
