@@ -1,6 +1,41 @@
 import torch
 import numpy as np
 
+def duplicate_faces(f):
+    """
+    duplicates *every* face in the mesh, with flipped orientation (and appends it to the end of the tensor)
+    note: will transfer to CPU if necessary with current implementation
+    :param f: faces of the mesh (Nx3)
+    :return: faces of the mesh with duplicated faces
+    """
+    if type(f) != np.ndarray:
+        new_faces = f.detach().cpu().numpy()
+    else:
+        new_faces = f
+    swapped_f = new_faces.copy()
+    swapped_f[:, [1, 2]] = swapped_f[:, [2, 1]]
+    f_new = np.concatenate([new_faces, swapped_f])
+    if type(f) != np.ndarray:
+        f_new = torch.tensor(f_new, dtype=f.dtype, device=f.device)
+    return f_new
+
+def remove_duplicate_faces(f):
+    """
+    remove duplicate faces from a mesh
+    note: will transfer to CPU if necessary with current implementation
+    :param f: faces of the mesh (Nx3)
+    :return: vertices and faces of the mesh without duplicate faces
+    """
+    if type(f) != np.ndarray:
+        f_new = f.detach().cpu().numpy()
+    else:
+        f_new = f
+    f_new = np.sort(f_new, axis=1)
+    f_new = np.unique(f_new, axis=0)
+    if type(f) != np.ndarray:
+        f_new = torch.tensor(f_new, dtype=f.dtype, device=f.device)
+    return f_new
+    
 def get_aspect_ratio(v: torch.Tensor, f: torch.Tensor):
     """
     measure aspect ratio of all triangles using: (circumradius / 2inradius)
@@ -144,8 +179,8 @@ def calc_edge_length(vertices: torch.Tensor, edges: torch.Tensor):
 def prepend_dummies(vertices: torch.Tensor, faces: torch.Tensor):
     """
     prepend dummy elements to vertices and faces to enable "masked" scatter operations
-    # V,D
-    # F,3 long
+    :param vertices (V,3)
+    :param faces (F,3) long
     """
     V, D = vertices.shape
     vertices = torch.concat((torch.full((1, D), fill_value=torch.nan, device=vertices.device), vertices), dim=0)
@@ -156,8 +191,9 @@ def prepend_dummies(vertices: torch.Tensor, faces: torch.Tensor):
 def remove_dummies(vertices: torch.Tensor, faces: torch.Tensor):
     """
     remove dummy elements added with prepend_dummies()
-    # V,D - first vertex all nan and unreferenced
-    # F,3 long - first face all zeros
+    :param vertices (V,3) - first vertex all nan and unreferenced
+    :param faces (F,3) long - first face all zeros
+    :return: V-1,D and F-1,3
     """
     return vertices[1:], faces[1:] - 1
 
@@ -184,28 +220,44 @@ def ray_sphere_intersection(sphere_origin, sphere_radius, ray_origin, ray_direct
 
 def vec2skew(v):
     """
+    returns the skew operator matrix given a vector
     :param v:  (3, ) torch tensor
     :return:   (3, 3)
     """
-    zero = torch.zeros(1, dtype=torch.float32, device=v.device)
-    skew_v0 = torch.cat([zero, -v[2:3], v[1:2]])  # (3, 1)
-    skew_v1 = torch.cat([v[2:3], zero, -v[0:1]])
-    skew_v2 = torch.cat([-v[1:2], v[0:1], zero])
-    skew_v = torch.stack([skew_v0, skew_v1, skew_v2], dim=0)  # (3, 3)
-    return skew_v  # (3, 3)
+    return batch_vec2skew(v[None, :])[0]
 
+def batch_vec2skew(v):
+    """
+    returns a batch of skew operator matrices given a batch of vectors
+    :param v:  (B, 3) torch tensor
+    :return:   (B, 3, 3)
+    """
+    zero = torch.zeros_like(v[:, 0:1])
+    skew_v0 = torch.cat([zero, -v[:, 2:3], v[:, 1:2]], dim=1)  # (B, 3)
+    skew_v1 = torch.cat([v[:, 2:3], zero, -v[:, 0:1]], dim=1)
+    skew_v2 = torch.cat([-v[:, 1:2], v[:, 0:1], zero], dim=1)
+    skew_v = torch.stack([skew_v0, skew_v1, skew_v2], dim=1)  # (B, 3, 3)
+    return skew_v  # (B, 3, 3)
+
+def batch_rotvec2mat(r: torch.Tensor):
+    """so(3) vector to SO(3) matrix
+    :param r: (3, ) axis-angle, torch tensor
+    :return:  (3, 3)
+    """
+    if r.ndim != 2:
+        raise ValueError("r must be 2D tensor.")
+    skew_r = batch_vec2skew(r)  # (3, 3)
+    norm_r = r.norm(dim=-1, keepdim=True)[:, :, None] + 1e-15
+    eye = torch.eye(3, dtype=torch.float32, device=r.device)[None, :]
+    R = eye + (torch.sin(norm_r) / norm_r) * skew_r + ((1 - torch.cos(norm_r)) / norm_r ** 2) * (skew_r @ skew_r)
+    return R
 
 def rotvec2mat(r: torch.Tensor):
     """so(3) vector to SO(3) matrix
     :param r: (3, ) axis-angle, torch tensor
     :return:  (3, 3)
     """
-    skew_r = vec2skew(r)  # (3, 3)
-    norm_r = r.norm() + 1e-15
-    eye = torch.eye(3, dtype=torch.float32, device=r.device)
-    R = eye + (torch.sin(norm_r) / norm_r) * skew_r + ((1 - torch.cos(norm_r)) / norm_r ** 2) * (skew_r @ skew_r)
-    return R
-
+    return batch_rotvec2mat(r[None, :])[0]
 
 def mat2rotvec(r: torch.Tensor):
     """SO(3) matrix to so(3) vector
@@ -222,11 +274,10 @@ def mat2rotvec(r: torch.Tensor):
     raise NotImplementedError("please verify this function implementation before usage")
     return rotvec * angle2
 
-
 def qvec2rotmat(qvec: np.array):
     """
-    converts a Quaternions to a rotation matrix
-    :param qvec: np array (4,)
+    converts a quaternions to a rotation matrix
+    :param qvec: np array (4,) xyzw
     :return: 3x3 np array
     """
     rot_mat = np.array([
@@ -239,35 +290,56 @@ def qvec2rotmat(qvec: np.array):
         [2 * qvec[3] * qvec[1] - 2 * qvec[0] * qvec[2],
          2 * qvec[2] * qvec[3] + 2 * qvec[0] * qvec[1],
          1 - 2 * qvec[1] ** 2 - 2 * qvec[2] ** 2]])
+    raise NotImplementedError("please verify this function implementation before usage")
     return rot_mat
 
-
 def rotmat2qvec(R: np.array):
     """
-    converts a rotation matrix to a Quaternions
+    converts a rotation matrix to a quaternion
     :param R: np array of size 3x3
-    :return: qvec (4,)
+    :return: qvec (4,) xyzw
     """
-    Rxx, Ryx, Rzx, Rxy, Ryy, Rzy, Rxz, Ryz, Rzz = R.flat
-    K = np.array([
-        [Rxx - Ryy - Rzz, 0, 0, 0],
-        [Ryx + Rxy, Ryy - Rxx - Rzz, 0, 0],
-        [Rzx + Rxz, Rzy + Ryz, Rzz - Rxx - Ryy, 0],
-        [Ryz - Rzy, Rzx - Rxz, Rxy - Ryx, Rxx + Ryy + Rzz]]) / 3.0
-    eigvals, eigvecs = np.linalg.eigh(K)
-    qvec = eigvecs[[3, 0, 1, 2], np.argmax(eigvals)]
-    if qvec[0] < 0:
-        qvec *= -1
-    return qvec
-
+    q = np.empty((R.shape[0], 4), dtype=R.dtype)
+    trace = np.trace(R)
+    expanded_trace = np.concatenate((np.diag(R), trace), axis=0)
+    choice = np.argmax(expanded_trace, axis=-1)
+    mask = choice != 3
+    i = choice[mask]
+    j = (i+1) % 3
+    k = (j+1) % 3
+    ii = np.concatenate((i, i), axis=1)
+    ij = np.concatenate((i, j), axis=1)
+    ik = np.concatenate((i, k), axis=1)
+    jk = np.concatenate((j, k), axis=1)
+    q[:, 0] = R[:, 2, 1] - R[:, 1, 2]
+    q[:, 1] = R[:, 0, 2] - R[:, 2, 0]
+    q[:, 2] = R[:, 1, 0] - R[:, 0, 1]
+    q[:, 3] = 1 + trace
+    q[mask, 0] = 1 - trace + 2*R[ii]
+    q[mask, 1] = R[np.flip(ij)] + R[ij]
+    q[mask, 2] = R[np.flip(ik)] + R[ik]
+    q[mask, 3] = R[np.flip(jk)] - R[jk]
+    raise NotImplementedError("please verify this function implementation before usage")
+    # Rxx, Ryx, Rzx, Rxy, Ryy, Rzy, Rxz, Ryz, Rzz = R.flat
+    # K = np.array([
+    #     [Rxx - Ryy - Rzz, 0, 0, 0],
+    #     [Ryx + Rxy, Ryy - Rxx - Rzz, 0, 0],
+    #     [Rzx + Rxz, Rzy + Ryz, Rzz - Rxx - Ryy, 0],
+    #     [Ryz - Rzy, Rzx - Rxz, Rxy - Ryx, Rxx + Ryy + Rzz]]) / 3.0
+    # eigvals, eigvecs = np.linalg.eigh(K)
+    # qvec = eigvecs[[3, 0, 1, 2], np.argmax(eigvals)]
+    # if qvec[0] < 0:
+    #     qvec *= -1
+    # return qvec
+    return q
 
 def qslerp(qa, qb, t):
     """
     interpolates between two quanternions
-    :param qa: 
-    :param qb:
-    :param t:
-    :return:
+    :param qa: first quaternions
+    :param qb: second quaternions
+    :param t: number between 0 and 1 indicating the interpolation factor
+    :return: b x 4 array of quaternions
     """
     qm = np.zeros_like(qa)
     cosHalfTheta = qa.dot(qb)
@@ -284,7 +356,6 @@ def qslerp(qa, qb, t):
     qm = qa*ratioA + qb*ratioB
     return qm
 
-
 def ray_ray_intersection(oa, da, ob, db):
     """
     returns point closest to both rays of form o+t*d,
@@ -379,14 +450,14 @@ def find_princple_componenets(v: torch.Tensor):
 def remove_unreferenced_vertices(v, f):
     """
     removes unreferenced vertices from a mesh
-    :param v: Vx3 torch tensor of vertices
-    :param f: Fx3 torch tensor of faces
-    :return: the new vertices and faces
+    :param v: Vx3 np array of vertices
+    :param f: Fx3 np array of faces
+    :return: the new vertices and faces, and a map from new to old indices (-1 if unreferenced)
     """
     referenced = np.zeros((v.shape[0]), dtype=np.bool)
     referenced[f.flatten()] = True
-    idx = -1 * np.ones((v.shape[0]), dtype=np.int32)
-    idx[referenced] = np.arange(np.sum(referenced), dtype=np.int32)
+    idx = -1 * np.ones((v.shape[0]), dtype=np.int64)
+    idx[referenced] = np.arange(np.sum(referenced), dtype=np.int64)
     f = idx[f.flatten()].reshape((-1, f.shape[1]))
     v = v[referenced]
     return v, f, idx
@@ -427,4 +498,17 @@ def edge_contraction(v, f, edge_to_contract, new_v_location):
     v[v2] = new_v_location
     # remove v1 from v
     v, f, _ = remove_unreferenced_vertices(v, f)
+    return v, f
+
+def clean_infinite_vertices(v, f):
+    """
+    removes vertices that are infinite / nan, and all their incident faces
+    :param v: V x 3 numpy array of vertices
+    :param f: F x 3 numpy array of faces
+    :return: the cleaned mesh
+    """
+    finite_mask = np.isfinite(v).any(axis=-1)  # finite mask
+    if ~np.all(finite_mask):
+        f = f[finite_mask[f].all(axis=-1)]  # remove faces with infinite vertices
+        v, f, _ = remove_unreferenced_vertices(v, f)  # update vertex indices
     return v, f
\ No newline at end of file
