@@ -3,6 +3,19 @@ import numpy as np
 from pathlib import Path
 from .core import to_8b, to_np
 from PIL import Image
+import json
+
+def write_to_json(data, dst):
+    """
+    writes data to json file
+    :param data: data to write
+    :param dst: path to save json file to
+    """
+    dst = Path(dst)
+    dst.parent.mkdir(parents=True, exist_ok=True)
+    with open(dst, "w") as f:
+        json.dump(data, f, indent=4, sort_keys=True)
+
 
 def save_animation(images, dst):
     """
@@ -24,7 +37,6 @@ def save_animation(images, dst):
     dst = Path(dst.parent, dst.stem)
     images[0].save(str(dst)+".gif", save_all=True, append_images=images[1:], optimize=False, duration=100, loop=0)
 
-
 def save_image(image, dst, force_grayscale: bool = False):
     """
     saves single image as png
@@ -39,7 +51,7 @@ def save_image(image, dst, force_grayscale: bool = False):
     dst.parent.mkdir(parents=True, exist_ok=True)
     save_images(image[None, ...], dst.parent, [dst.name], force_grayscale)
 
-def save_images(images, dst, file_names: list = [], force_grayscale: bool = False):
+def save_images(images, dst, file_names: list = [], force_grayscale: bool = False, overwrite: bool = True):
     """
     saves images as png
     :param images: (b x H x W x C) tensor
@@ -58,6 +70,7 @@ def save_images(images, dst, file_names: list = [], force_grayscale: bool = Fals
     if file_names:
         if images.shape[0] != len(file_names):
             raise ValueError("Number of images and length of file names list must match")
+        file_names = [Path(x).stem for x in file_names]  # remove suffix
     for i, image in enumerate(images):
         if force_grayscale or images.shape[-1] == 1:
             if images.shape[-1] == 3:
@@ -66,19 +79,28 @@ def save_images(images, dst, file_names: list = [], force_grayscale: bool = Fals
         else:
             pil_image = Image.fromarray(image)
         if file_names is not None:
-            file_names = [Path(x).stem for x in file_names]  # remove suffix
-            pil_image.save(str(Path(dst, "{}.png".format(file_names[i]))))
+            cur_dst = Path(dst, "{}.png".format(file_names[i]))
+            if not overwrite:
+                if cur_dst.exists():
+                    continue
+            pil_image.save(str(cur_dst))
         else:
-            pil_image.save(str(Path(dst, "{:05d}.png".format(i))))
+            cur_dst = Path(dst, "{:05d}.png".format(i))
+            if not overwrite:
+                if cur_dst.exists():
+                    continue
+            pil_image.save(str(cur_dst))
 
-def load_image(path, to_float=False, channels_last=True, return_paths=False, to_torch=False, device=None):
+def load_image(path, to_float=False, channels_last=True, to_torch=False, device=None, resize_wh=None, as_grayscale=False):
     """
-    loads images from a folder or a single image from a file
-    :param path: path to folder with images / file
-    :param to_float: if True, converts images to float
+    loads an image from a single file
+    :param path: path to file
+    :param to_float: if True, converts image to float
     :param return_paths: if True, returns a list of file paths
     :param to_torch: if True, returns a torch tensor
     :param device: device to load tensor to
+    :param resize_wh: a tuple (w, h) to resize the image using nearest neighbor interpolation
+    :param as_grayscale: if True, loads image as grayscale by averaging over the channels
     :return: (b x H x W x 3) tensor, and optionally a list of file names
     """
     path = Path(path)
@@ -87,42 +109,50 @@ def load_image(path, to_float=False, channels_last=True, return_paths=False, to_
     if path.is_dir():
         raise FileNotFoundError("Path must be a file")
     elif path.is_file():
-        if return_paths:
-            image, name = load_images(path, to_float=to_float, channels_last=channels_last, return_paths=return_paths, to_torch=to_torch, device=device)
-            return image[0], name[0]
-        else:
-            image = load_images(path, to_float=to_float, channels_last=channels_last, return_paths=return_paths, to_torch=to_torch, device=device)
-            return image[0]
+        image = load_images(path, to_float=to_float, channels_last=channels_last, return_paths=False, to_torch=to_torch, device=device, resize_wh=resize_wh, as_grayscale=as_grayscale)
+        return image[0]
 
-def load_images(path, to_float=False, channels_last=True, return_paths=False, to_torch=False, device=None):
+def load_images(source, to_float=False, channels_last=True, return_paths=False, to_torch=False, device=None, resize_wh=None, as_grayscale=False):
     """
     loads images from a list of paths, a folder or a single file
-    :param path: path to folder with images / file
-    :param to_float: if True, converts images to float
+    :param source: path to folder with images / path to image file / list of paths
+    :param to_float: if True, converts images to float (and normalizes to [0, 1])
     :param return_paths: if True, returns a list of file paths
     :param to_torch: if True, returns a torch tensor
     :param device: device to load tensor to
-    :return: (b x H x W x 3) tensor, and optionally a list of file names
+    :param resize_wh: a tuple (w, h) to resize all images using nearest neighbor interpolation
+    :param as_grayscale: if True, loads images as grayscale by averaging over the channels
+    :return: (b x H x W x C) tensor, and optionally a list of file names
     """
     supported_suffixes = [".png", ".jpg", ".jpeg"]
-    if type(path) == list:
+    if type(source) == list or type(source) == tuple or type(source) == np.ndarray:
         images = []
         file_paths = []
-        for p in path:
-            if not Path(p).exists():
+        for p in source:
+            p = Path(p)
+            if not p.exists():
                 raise FileNotFoundError("Path does not exist")
             if p.suffix in supported_suffixes:
-                images.append(np.array(Image.open(str(p))))
+                im = Image.open(str(p))
+                if resize_wh is not None:
+                    im = im.resize(resize_wh)
+                images.append(np.array(im))
                 file_paths.append(p)
         images = np.stack(images, axis=0)
-        if not channels_last:
+        if as_grayscale and images.ndim == 4:
+            images = images.mean(axis=-1).astype(np.float32)
+        if not channels_last and images.ndim == 4:
             images = np.moveaxis(images, -1, 1)
         if to_float:
             images = images.astype(np.float32) / 255
-        if to_torch and device is not None:
+        else:
+            images = images.astype(np.uint8)
+        if to_torch:
+            if device is None:
+                device = torch.device("cpu")
             images = torch.tensor(images, device=device)
     else:
-        path = Path(path)
+        path = Path(source)
         if not path.exists():
             raise FileNotFoundError("Path does not exist")
         if path.is_dir():
@@ -130,23 +160,41 @@ def load_images(path, to_float=False, channels_last=True, return_paths=False, to
             file_paths = []
             for image in sorted(path.iterdir()):
                 if image.suffix in supported_suffixes:
-                    images.append(np.array(Image.open(str(image))))
+                    im = Image.open(str(image))
+                    if resize_wh is not None:
+                        im = im.resize(resize_wh)
+                    images.append(np.array(im))
                     file_paths.append(image)
             images = np.stack(images, axis=0)
-            if not channels_last:
+            if as_grayscale and images.ndim == 4:
+                images = images.mean(axis=-1).astype(np.float32)
+            if not channels_last and images.ndim == 4:
                 images = np.moveaxis(images, -1, 1)
             if to_float:
                 images = images.astype(np.float32) / 255
-            if to_torch and device is not None:
+            else:
+                images = images.astype(np.uint8)
+            if to_torch:
+                if device is None:
+                    device = torch.device("cpu")
                 images = torch.tensor(images, device=device)
         elif path.is_file():
-            images = np.array(Image.open(str(path)))
-            if not channels_last:
+            im = Image.open(str(path))
+            if resize_wh is not None:
+                im = im.resize(resize_wh)
+            images = np.array(im)
+            if as_grayscale and images.ndim == 3:
+                images = images.mean(axis=-1).astype(np.float32)
+            if not channels_last and images.ndim == 3:
                 images = np.moveaxis(images, -1, 0)
             if to_float:
                 images = images.astype(np.float32) / 255
+            else:
+                images = images.astype(np.uint8)
             file_paths = [path]
-            if to_torch and device is not None:
+            if to_torch:
+                if device is None:
+                    device = torch.device("cpu")
                 images = torch.tensor(images, device=device)
             images = images[None, ...]
     if return_paths:
