@@ -1,7 +1,7 @@
 import numpy as np
 from PIL import Image, ImageDraw, ImageFont
 from scipy import interpolate, spatial
-from .core import to_8b, to_float, to_hom, homogenize, broadcast_batch
+from .core import to_8b, to_float, to_hom, homogenize, broadcast_batch, map_range
 from .structures import get_gizmo_coords
 
 def alpha_compose(images, bg_color=None):
@@ -183,4 +183,87 @@ def interpolate_multi_channel(image: np.ndarray, mask: np.ndarray):
     for channel in range(image.shape[-1]):
         interpolated_channel = interpolate_single_channel(image[:, :, channel], mask)
         interpolated[:, :, channel] = interpolated_channel
-    return interpolated
\ No newline at end of file
+    return interpolated
+
+def image_grid(images, rows, cols):
+    """
+    :param images: list of images
+    :param rows: number of rows
+    :param cols: number of cols
+    :return: grid image
+    """
+    if images.ndim != 4:
+        raise ValueError("images must be a 4D array")
+    if len(images) != rows * cols:
+        raise ValueError("number of images must be equal to rows * cols")
+    tmp = images.reshape(rows, cols, images.shape[1], images.shape[2], -1)
+    result = tmp.transpose(0, 2, 1, 3, 4).reshape(rows * images.shape[1], cols * images.shape[2], -1)
+    return result
+
+def resize_images_naive(images, H, W, channels_last=True, mode="mean"):
+    """
+    resize images to output_size, but only if the output size has a common divisor of the input size
+    :param images: numpy array of images (N x H x W x C)
+    :param H: output height size that has a common divisor with the input height size
+    :param W: output width size that has a common divisor with the input width size
+    :param channels_last: if True, the images are provided in channels last format (and so will the output)
+    :param mode: one of "max", "mean"
+    :return: np array of resized images
+    """
+    if images.ndim != 4:
+        raise ValueError("images must be a 4D array")
+    if images.shape[1] != images.shape[2]:
+        raise ValueError("images must be square")
+    if not channels_last:
+        raise NotImplementedError("only channels last is supported")
+    input_size = np.array(images.shape[1:3])
+    output_size = np.array([H, W])
+    bin_size = input_size // output_size
+    if mode == "max":
+        small_images = images.reshape((images.shape[0], output_size[0], bin_size[0], output_size[1], bin_size[1], 3)).max(4).max(2)
+    elif mode == "mean":
+        small_images = images.reshape((images.shape[0], output_size[0], bin_size[0], output_size[1], bin_size[1], 3)).mean(4).mean(2)
+        small_images = small_images.astype(np.uint8)
+    else:
+        raise ValueError("mode must be one of 'max', 'mean'")
+    return small_images
+
+def change_brightness(input_img, brightness=255):
+    """
+    changes brightness of an image
+    :param input_img the numpy image
+    :param brightness a number between -255 to 255 (0=no change)
+    :return the new image
+    """
+    brightness = map_range(brightness, 0, 510, -255, 255)
+
+    if brightness != 0:
+        if brightness > 0:
+            shadow = brightness
+            highlight = 255
+        else:
+            shadow = 0
+            highlight = 255 + brightness
+        alpha_b = (highlight - shadow)/255
+        gamma_b = shadow
+    else:
+        alpha_b = 1
+        gamma_b = 0
+    return input_img*alpha_b + gamma_b
+
+def change_contrast(input_img, contrast=127):
+    """
+    changes brightness of an image
+    :param input_img the numpy image
+    :param brightness a number between -127 to 127 (0=no change)
+    :return the new image
+    """
+    contrast = map_range(contrast, 0, 254, -127, 127)
+    if contrast != 0:
+        f = float(131 * (contrast + 127)) / (127 * (131 - contrast))
+        alpha_c = f
+        gamma_c = 127*(1-f)
+    else:
+        alpha_c = 1
+        gamma_c = 0
+    return input_img*alpha_c + gamma_c
\ No newline at end of file
